package com.example.afrishop_v3.controllers;
import com.example.afrishop_v3.base.Result;
import com.example.afrishop_v3.enums.ResultCodeEnum;
import com.google.api.client.util.IOUtils;
import com.google.cloud.vision.v1.*;
import com.google.cloud.vision.v1.Feature.Type;
import com.google.protobuf.ByteString;
import com.google.protobuf.Descriptors;
import com.google.protobuf.GeneratedMessageV3;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
@RequestMapping("search/image")
public class ImageSearchController extends Controller{
    // Imports the Google Cloud client library



    @GetMapping
    public Result main() throws Exception {
        // Initialize client that will be used to send requests. This client only needs to be created
        // once, and can be reused for multiple requests. After completing all of your requests, call
        // the "close" method on the client to safely clean up any remaining background resources.
        Result errorDecoding = new Result(ResultCodeEnum.VALIDATE_ERROR.getCode(), "Error decoding");
        authExplicit();
        try (ImageAnnotatorClient vision = ImageAnnotatorClient.create()) {

            // The path to the image file to annotate
            String fileName = "./resources/wakeupcat.jpg";


            // Reads the image file into memory
            //Path path = Paths.get(fileName);

            InputStream in = new ClassPathResource("Blue_Tshirt.jpg").getInputStream();
            byte[] data = IOUtils.serialize(in);
            //byte[] data = Files.readAllBytes(path);
            ByteString imgBytes = ByteString.copyFrom(data);

            // Builds the image annotation request
            List<AnnotateImageRequest> requests = new ArrayList<>();
            Image img = Image.newBuilder().setContent(imgBytes).build();
            Feature feat = Feature.newBuilder().setType(Type.OBJECT_LOCALIZATION).build();
            AnnotateImageRequest request =
                    AnnotateImageRequest.newBuilder().addFeatures(feat).setImage(img).build();
            requests.add(request);

            // Performs label detection on the image file
            BatchAnnotateImagesResponse response = vision.batchAnnotateImages(requests);
            List<AnnotateImageResponse> responses = response.getResponsesList();

            for (AnnotateImageResponse res : responses) {
                if (res.hasError()) {
                    System.out.format("Error: %s%n", res.getError().getMessage());
                    return errorDecoding;
                }
                List<Map<Descriptors.FieldDescriptor, Object>> collect = res.getLocalizedObjectAnnotationsList().stream().map(GeneratedMessageV3::getAllFields).collect(Collectors.toList());
//                for (LocalizedObjectAnnotation annotation : res.getLocalizedObjectAnnotationsList()) {
//                    annotation
//                            .getAllFields()
//                            .forEach((k, v) -> System.out.format("%s : %s%n", k., v.toString()));
//                }
                return new Result<>(collect);
            }
        }
        return errorDecoding;
    }
}
